@@ -16,11 +16,10 @@ public:
 		: exp1 (e1)
 		, exp2 (e2)
 	{}
-	void split (CharSet &);
-	void calcSize (Char *);
+	void split (std::set<uint32_t> &);
+	uint32_t calc_size() const;
 	uint32_t fixedLength ();
-	uint32_t compile (Char *, Ins *);
-	void decompile ();
+	nfa_state_t *compile(nfa_t &nfa, nfa_state_t *n);
 	void display (std::ostream & o) const;
 	friend RegExp * mkAlt (RegExp *, RegExp *);
 
