@@ -34,7 +34,7 @@ struct Spec
 		}
 		else
 		{
-			addl (r);
+			add (r);
 			return true;
 		}
 	}
@@ -43,11 +43,6 @@ struct Spec
 		rules[r->rank].line = r->loc.line;
 		re = mkAlt (re, r);
 	}
-	void addl (RuleOp * r)
-	{
-		rules[r->rank].line = r->loc.line;
-		re = mkAlt (r, re);
-	}
 	void clear ()
 	{
 		re = NULL;
